package CPANPLUS::Dist::Pacman;

=pod

=head1 NAME

CPANPLUS::Dist::Pacman

=head1 SYNOPSIS

    ### read the code & comments
    perldoc -m CPANPLUS::Dist::Pacman

=head1 DESCRIPTION

C<CPANPLUS::Dist::Pacman> is a distribution class to create C<ArchLinux> 
packages from C<CPAN> modules, and all it's dependencies. This allows
you to have automatic access to the most recent copies of all C<CPAN> modules,
and to install them with C<pacman>.

=cut


use strict;

### inherit from CPANPLUS::Dist
use Exporter;
#use vars	 qw[@ISA $VERSION @EXPORT_OK];
our @ISA = 		 qw[CPANPLUS::Dist Exporter];
our $VERSION =	 '0.3.6';
our @EXPORT_OK = qw[archpkgname running_archlinux pacman_installed make_abscache %perl_abs %ARCHPKGCONF];

### for I18N ###
use Locale::Maketext::Simple    Class => 'CPANPLUS', Style => 'gettext';

### for error handling, which gets added to the error stack
### use as: cp_error(loc("Something is wrong: %1", $error));
### or:     cp_msg(loc("FYI: %1 and %2", $foo, $bar), $verbose);
use CPANPLUS::Error;

### common constants ###
use CPANPLUS::Internals::Constants;

# FIXME arent these loaded by CPANPLUS anyway?
use FileHandle;
use File::Basename;
use File::Find;
use File::Path;
use File::Spec;
use Cwd;
use IPC::Cmd					qw[run can_run];
use Archive::Tar;
### verbose errors on param checking ###
#use Params::Check 				qw[check];
#local $Params::Check::VERBOSE = 1;


use YAML qw[DumpFile LoadFile];

### Configuration variables ###

our %ARCHPKGCONF;

my %default_archpkgconf = (
    'version' => $VERSION,
    'makepkg' => 1, 
	'makepkgflags' => '-de',
	'clean' => 0,
	'maketest' => 0,
	'forceinstalled' => 0,
    'usemeta' => 1,
    'prereqs' => 1,
    'basedir' => '/var/abs/local/cpan', 
    'abscachefile' => 'perl_abs.yml', 
    'absdirs' => [ '/var/abs/extra', '/var/abs/community' ]
    ) ;
    
my %storedconfig = ();
my $cpanplusdir = File::Spec->catdir($ENV{HOME},'.cpanplus') || die "$!";
my $configfile = File::Spec->catfile($cpanplusdir, 'cpan4pacman.yml') || die "$!" ;

unless ( -d $cpanplusdir ) {
	mkdir($cpanplusdir);
	print <<"EOF"
**********************************************************
$cpanplusdir does not exist, so it is possible you have 
not configured CPANPLUS yet, in which case you should do 
so now, for instance to set your nearest CPAN mirrors. 
Type 'cpanp' to enter the cpanplus shell and type 
's reconfigure' from there. We suggest you choose to set 
the config file as '$cpanplusdir/config'. 
Of course you can also use the system-wide config file
(/usr/lib/perl5/site_perl/current/CPANPLUS/Config.pm).
*******************************************************
EOF
};

###NOT NECESSARY FOR NOW:
#unless ( exists $ENV{PERL5_CPANPLUS_CONFIG} ) {
#    if ( -f File::Spec->catfile($cpanplusdir, 'config') ) {
#        $ENV{PERL5_CPANPLUS_CONFIG} = File::Spec->catfile($cpanplusdir, 'config') 
#    } else {
#        $ENV{PERL5_CPANPLUS_CONFIG} =  '/usr/lib/perl5/site_perl/current/CPANPLUS/Config.pm'
#   }
#}; 

if ( -f $configfile ) {
     my $storedconfigref = LoadFile($configfile) || die "Cannot load $configfile: $!";
     %storedconfig = %$storedconfigref;
     delete $storedconfig{'version'};
     # TODO warning if new version is detected?
	 %ARCHPKGCONF = (%default_archpkgconf, %storedconfig);
}
else { 
	%ARCHPKGCONF = %default_archpkgconf;
	print ">>> NB: Your config file for cpan4pacman is\n>>> $configfile\n";
};

# Bugfix: delete the keys 'abs' (disabled) and 'absachefile' (was a typo).
delete @ARCHPKGCONF{qw[abs absachefile]};

# this makes sure that new keys in the config hash will be saved
# when cpan4pacman is updated to a new version
DumpFile($configfile, \%ARCHPKGCONF) || die "Can't save $configfile: $!"; 

my $abscachefile = $ARCHPKGCONF{'abscachefile'} ;

#if (defined $ARCHPKGCONF{'abscachefile'}) {
#    $abscachefile = $ARCHPKGCONF{'abscachefile'} 
#}
#else { 
#    cp_error( loc("'abscachefile' is not defined in your config file ($configfile)") ) 
#}; 

my $abscachepath = File::Spec->catfile($cpanplusdir, $abscachefile) || die "$!"; 

my $perl_absref;
our %perl_abs;

unless ( -f $abscachepath ) {
    make_abscache()
};

$perl_absref = LoadFile($abscachepath) || 
        cp_error(loc("Could not load the ABS cache %1", $abscachepath)) ;
%perl_abs = %$perl_absref;



###################################################################

### sub called just after the CPANPLUS::Dist object is created ###
### it does nothing really ;^)


sub init {
    my $self    = shift;
    my $status  = $self->status;

    ### minimally required accessors are already created for you:
    # qw[prepared created installed uninstalled dist distdir]
    #
    #XXX $status->mk_accessors(qw[make_prereqs]); 

    return 1;
};

sub make_abscache { 
    my $absdirsref = $ARCHPKGCONF{'absdirs'}; 
		#   default is [ '/var/abs/system', '/var/abs/extra', '/var/abs/community' ] ;
    my @absdirs = @$absdirsref;
	
	# this will populate the perl_abs hash :
	find(\&pkgbuildwanted, @absdirs); 
    
	unless ( DumpFile($abscachepath, \%perl_abs) ) {
        cp_error(loc("Could not create ABS cache %1", $abscachepath)) ;
        return ;
    } ;
    cp_msg(loc("ABS cache saved to %1", $abscachepath));
}

sub pkgbuildwanted {
	my $ffdir = $File::Find::dir;
	$ffdir =~ s|.+/([^/]+)|$1|;
	($ffdir =~ /perl/ or $ffdir eq 'mp3-info') && $_ eq 'PKGBUILD' && parse_pkgbuild()
}

sub parse_pkgbuild {
	my $fh = new FileHandle;
	my ($pkg, $srcname, $ver, $rel, $desc, $url, $src,$cpan_pkg, $rest);
	$fh->open("< PKGBUILD") or die "Cannot open the PKGBUILD for $File::Find::dir";
	while (<$fh>) {
		if (/^pkgname=/) {
			(my $tmp, $pkg) = split /=/, $_ ;
			chomp($pkg);
		}
		elsif (/^_?(src|real|cpan)name=/) {
			(my $tmp, $srcname) = split /=/, $_ ;
			chomp($srcname);
		}
		elsif (/^pkgver=/) {
			(my $tmp, $ver) = split /=/, $_ ;
			chomp($ver);
		}
		elsif (/^pkgrel=/) {
			(my $tmp, $rel) = split /=/, $_ ;
			chomp($rel);
		}
		elsif (/^url=/) {
			(my $tmp, $url) = split /=/, $_ ;
			chomp($url);
			$url =~ s/\${?pkgver}?/$ver/g;
			$url =~ s/\${?pkgname}?/$pkg/g;
			$url =~ s/\${?_?(src|real|cpan)name}?/$srcname/g if defined $srcname;
            $url =~ s/^['"\s]\+//g;
            $url =~ s/['"\s]\+$//g;
		}
		elsif (/^pkgdesc=/) {
			(my $tmp, $desc) = split /=/, $_ ;
			chomp($desc);
            $desc =~ s/^['"\s]\+//g;
            $desc =~ s/['"\s]\+$//g;
		}
		elsif (/^source=/) {
			(my $tmp, $src) = split /=/, $_ ;
			chomp($src);			
			$src =~ s/\${?pkgver}?/$ver/g;
			$src =~ s/\${?pkgname}?/$pkg/g;
			$src =~ s/\${?_?(src|real|cpan)name}?/$srcname/g if defined $srcname;
			($cpan_pkg,$rest) = ( $src =~ m#/([^/ ]+)-([^/ ]+)\.(tar\.gz|tgz|zip|tar\.bz2)# );
		}
		else {next};				
	};
	$perl_abs{$cpan_pkg} = {arch_package => $pkg, version => $ver, relversion => $rel, description => $desc, url => $url, source => $src};
};

my $pacman = can_run("pacman") ;

sub running_archlinux {
	my $flag = ( -e '/etc/arch-release' ) ;
	if (defined ($pacman)) {
		$flag++
	}; 
	return $flag;
}

sub pacman_installed {
	my $pkg = shift;
	my $cmd = "$pacman -Q $pkg";
	my @pacbuf = run( command => $cmd, verbose => 0 );
	return $pacbuf[0]; 
}

### sub called to see if it is possible to create this type of dist
### on the environment we're running in. should warn about why this
### dist cannot be run on this environment. return true on possible,
### false otherwise.
#
sub format_available {
    my $flag ;
	unless ( -e '/etc/arch-release' ) {
		cp_error(loc("The file /etc/arch-release was not found. Are we running ArchLinux?"));
		$flag++;
	}
	unless ( defined($pacman) ) { 
		cp_error(loc("pacman was not found :( Are we running ArchLinux?"));
		$flag++
	}
    for my $pkg (qw[make gcc fakeroot findutils]) {
        unless( pacman_installed($pkg) ) {
            cp_error(loc("'%1' is a required package. Please install it first",
                      $pkg));
            $flag++;
        }
    }
    return $flag ? 0 : 1;
}

### Sub to transform a CPAN package name into an Arch Linux pkg name
#
sub archpkgname {
	# argument is $modobj->package_name
	my $pkg = shift;
	$pkg = lc $pkg ;  #tr/A-Z/a-z/
	$pkg =~ s/mod_perl/modperl/;
	$pkg =~ s/_/-/g; # some weird stuff in Acme::* namespace
	# Package names ending in -perl such as libwww-perl should be distributed 
	# as perl-libwww not perl-libwww-perl
	# NB: when 'perl' is in the middle we leave it: dce-perl-rpc -> perl-dce-perl-rpc 
	#     even though dce-perl -> perl-dce
	# 	  EXCEPTION : glade-perl-two -> glade-two
	$pkg =~ s/^glade-perl-two$/glade-two/;
	$pkg =~ s/^aceperl$/ace/;
	# But there are some exceptions where we leave it to avoid confusion,
	# because both packages foo-bar-perl and foo-bar exist:
	#		 >>> TODO : only remove -perl from foo-bar-perl when no CPAN package exists with the pattern foo-bar
	unless ( $pkg =~ /^(class-phrasebook|compress-zlib|devel-caller|heap-simple|inline|log-dispatch|mime-base64|net-ssh|syntax-highlight|template-plugin)-perl$/ ) {
		$pkg =~ s/^(.+)-perl$/$1/;
	};
	if ( $pkg =~ m/^perl/) { # perl-ldap, perl-gps, perl-gpsdata, perl-lisp, etc and all Perl::* modules
		return $pkg
	} else {
	return "perl-$pkg"
	};
}


### Check if CPAN module is provided from a package in the Arch Linux repositories
sub package_is_in_arch_repos {
	my $obj = shift;
	my $pkg = $obj->package_name;
 	exists ($perl_abs{$pkg}) ? return 1 : return 0;
}

# /////////////////////////////////////////////////////////////////////////
# 

sub prepare { 
    ### just in case you already did a create call for this module object
    ### just via a different dist object
    my $dist = shift;
    my $self = $dist->parent;
	# next lines comes from Deb.pm : check whether the Module uses Build.PL or Makefile.PL
    my $dist_cpan   = $self->status->dist_cpan;
    $dist    = $self->status->dist   if      $self->status->dist;
    $self->status->dist( $dist )     unless  $self->status->dist;

    my $cb   = $self->parent;
    my $conf = $cb->configure_object;
    
    my $force = $conf->get_conf('force');
    my $verbose = $conf->get_conf('verbose');
    my $skiptest = $conf->get_conf('skiptest');

	my $pkgname = $self->package_name;
    my $archpkgname = archpkgname($pkgname) ; 
    
	my $tarball =  $self->status->fetch; 
	
	my $makepkgsrcdir = $self->status->extract; 
    my $makepkgdir = File::Spec->catdir( $makepkgsrcdir, '..', '..' );
	
	my $srcpkg			= $self->package;
    
    ## This is not always the case!!! 
    #  Sometimes tarballs for Foo-Bar-0.01 will be packed in dir Foo-Bar 
    my $srcpkgpath ; #     = File::Spec->catfile($makepkgdir, $srcpkg);
    { 
      my @list = Archive::Tar->list_archive($tarball) or cp_error( "cannot get the list of files in $tarball ") ;
      my $tarballdir = $list[0] ;
      $srcpkgpath       = File::Spec->catfile($makepkgsrcdir, $tarballdir);
    }

	unless ($srcpkg eq basename($tarball) ) { 
		cp_error( loc("%1 should be equal to basename of %2", $srcpkg, $tarball) ) 
	};
	
	system("ln -s $tarball $makepkgdir/") unless -e $srcpkgpath; #FIXME catch error...
    
	my $installtype	= $self->status->installer_type; 
    ## returns "CPANPLUS::Dist::Build" or "CPANPLUS::Dist::MM"
	
    ## my $maintainer      = $conf->get_conf('email'); 
    # we get it by parsing /etc/makepkg.conf instead
    chomp( my $maintainer = `source /etc/makepkg.conf && echo \$PACKAGER` ) ; 

	my $pkgdesc         = $self->description || $self->module;
    #   if no description is available through CPANPLUS, fetch one as follows:
    if ($pkgdesc eq $self->module) {
        print ">>> No description available, let's get one from search.cpan.org\n";
        # print ">>> (For that we need LWP::Simple and XML::XPath)\n";
        my $modmissing = 0;
        foreach ('LWP::Simple', 'XML::XPath') { 
            unless ( eval "require $_ ; 1" ) {
                cp_msg( loc("Module $_ not available!") ) ; 
                $modmissing++ ;
             }
        } ;
        if ($modmissing == 0) {
            my $url = "http://search.cpan.org/search?mode=dist&format=xml&query=$pkgname";
            require LWP::Simple; 
            my $xmlinfo = LWP::Simple::get($url);
            if ($xmlinfo) {
                ## TODO skip unless $xmlinfo
                require XML::XPath;
                my $xp = XML::XPath->new( xml => $xmlinfo );
                my $xpath = '//*[name="' . $pkgname . '"]/description';
                my $xpdesc = $xp->find($xpath);
                $pkgdesc = $xpdesc->string_value;
            } else {
                cp_msg( loc("Cannot reach search.cpan.org") )
            }
        } else {
             cp_msg( loc("For LWP::Simple install perl-libwww;\nfor XML::XPath install perl-xml-xpath") ) 
        }
    };
    
    $pkgdesc            =~ s/"//g;
	my $pkgver			= $self->package_version;
    my $pkgver_has_prefix = $pkgver =~ /^v/i ? 1 : 0 ;
    my $cpanver         = $pkgver;
    my $pkgverstring    = 'pkgver';
    if ($pkgver_has_prefix) {
        $pkgver         =~ s/^v//i;
        $pkgverstring   = '_cpanver';
    } ;
    my $pkgverline      = 
        $pkgver_has_prefix ? "pkgver=$pkgver\n_cpanver=$cpanver" : "pkgver=$pkgver" ;    
    my $pkgext          = $self->package_extension;
	my $pkgrel 			= '1'; #TODO make this an option? so we can call with --pkgrel=2 ?
	my $canonsrcpath	= $self->path;
	my $srcpath = File::Spec->catfile($canonsrcpath, $srcpkg);
	$canonsrcpath =~ s|^.+authors/|authors/|; # clean the path when using local tarballs downloaded by cpanplus
	my @srcpath			= split (/\//, $canonsrcpath);
	my $srcauthorid		= pop @srcpath;
#    my $_srcpkg         = $srcpkg;
#       $_srcpkg         =~ s/$pkgver/\$pkgver/;
	my $pkgsource		= "http://www.cpan.org/$canonsrcpath/\${_cpanname}-\${$pkgverstring}.$pkgext";
	my $pkgurl			= "http://search.cpan.org/~$srcauthorid/\${_cpanname}";
	my $pkgmd5;
	if ($ARCHPKGCONF{'archive'}) {
		$pkgmd5 = `md5sum $srcpath | cut -d' ' -f1` || '?' ;
		chomp($pkgmd5);
	} else {
		$pkgmd5	= $self->status->checksum_value || '?';
	};
	if ($pkgmd5 eq '?') { 
		cp_msg("Could not determine the md5sum of the tarball", $verbose)
	};
    
	my $prereqs	= $self->status->prereqs ;
    
    my %makeprereqs = ();
    
#    my $ignoreprereqs = ! $ARCHPKGCONF{'prereqs'};

#    unless ($ignoreprereqs) 
    {
        chdir($makepkgsrcdir);
		
        my $usemeta = $ARCHPKGCONF{'usemeta'};
        my $metayml = 'META.yml';
        my %p;
        
        if ( ( $usemeta or ($installtype =~ /build/i) ) 
                and ( -e $metayml ) ) {
            my $metaymlnode = LoadFile($metayml);
            my $reqs = $metaymlnode->{requires} if (exists $metaymlnode->{requires});
            my $breqs = $metaymlnode->{build_requires} if (exists $metaymlnode->{build_requires});
            %p = %$reqs if $reqs;
            %makeprereqs = %$breqs if $breqs;
        }
        else {
            if ($usemeta) { cp_msg(loc("META.yml not found.")) };
            cp_msg( loc("Calling perl Makefile.PL to get prereqs from dummy Makefile"), $verbose);
            my $captured;
            ## FIXME : this will stall in the case of interactive setups... (eg Template-Toolkit)
            my $cmd = ['perl', '-MCPANPLUS::Internals::Utils::Autoflush', 'Makefile.PL'];
            unless (scalar run( command => $cmd, 
                                verbose => 0, 
                                buffer => \$captured ) 
                        ) {
                cp_error(loc("cannot run perl Makefile.PL: %1 %2", $!, $captured));
            };
            open (TMPFH, "< Makefile") or cp_error( loc("Cannot open Makefile: $!"));
			while (<TMPFH>) {
				next unless /PREREQ_PM/; 
				unless (my ($found) = m/^\#\s+PREREQ_PM\s+=>\s+{\s+(.+)\s+}/) {
						cp_msg(loc("Could not find dependencies in the Makefile"), $verbose)
				} else {
					my @items = split /,\s+/, $found;

					foreach my $item (@items) {
						my ($a, $b) = ($item =~ /([\w\:]+)=>(?:q\[(.*?)\])/);
						$p{$a} = $b;
				   }
			   }
               last;
            };
            close TMPFH;
            cp_msg( loc("Moving dummy Makefile to Makefile.dummy"), $verbose);
            my $captured2;
            my $cmd2 = ['mv', 'Makefile', 'Makefile.dummy'];
            unless (scalar run( command => $cmd2, 
                                verbose => 0, 
                                buffer => \$captured2 ) 
                        ) {
                cp_error(loc("cannot rename Makefile to Makefile.dummy", $!, $captured));
            };

        }
    $self->status->prereqs( \%p );
        
	};
	
	chdir($makepkgdir);

    my $number_of_xs_files = `find src -name '*.xs' -print | wc -w` ;
    my $number_of_c_files = `find src -name '*.c' -print | wc -w` ;

    my $pkgarch = '\'any\'' ;

    if ($number_of_xs_files + $number_of_c_files > 0) {
        $pkgarch = '\'i686\' \'x86_64\''
    }

	if ( -f "PKGBUILD") {
			if ( $force ) {
				cp_msg( loc("Existing PKGBUILD for %1 backuped as PKGBUILD.old",
						 $archpkgname) );
				system("mv PKGBUILD PKGBUILD.old") ;	 
			} else { 
				cp_error( loc("PKGBUILD already exists! Use --force to overwrite") );
				$dist->status->prepared(0);
				return;
			}
	};

	unless ( open (PB, ">PKGBUILD") ) {
		cp_error( loc( "Could not open %1/PKGBUILD for writing: %2",
					 $makepkgdir, $! ) );
		$dist->status->prepared(0);
		return;
	};
	
    my @depends;
    my @archdepends;

    {   my $deps = $self->status->prereqs;
        my @depnames = sort keys %$deps;

        #remove "perl" when there is at least one more dependency (that depends on perl)
        if ($#depnames > 0 and exists $deps->{perl}) {
            delete($deps->{perl}) 
        };
        
        # when no dependencies are provided, add at least "perl"
        if ($#depnames == -1) {
            my $tmp = "perl";
            $tmp .= ">=5.10.0" if $] < 5.01;
            push @archdepends, $tmp
        } 
        else {
            for my $dep ( @depnames ) {
                #  make $depver a string if defined, empty if not
                my $depver = $deps->{$dep};
                my $depverf = $depver ? ">=$depver" : "" ;
              
                if ($dep eq 'perl') {
                    $depverf = ">=5.10.0" unless $] < 5.01;
                    push @archdepends, "perl$depverf"; 
                    next;
                };
                
                ### no point in listing prereqs that are IN the perl core
                ### themselves
                # next if $obj->module_is_supplied_with_perl_core; FIXME BROKEN!
                my $ver = $];
                $ver =~ s/0+$//;
                next if ( $Module::CoreList::version{$ver}{$dep} );

                ### We skip processing 'Test::*' modules if $skiptest is true
                next if ($skiptest and $dep =~ m/^Test::/) ;

                my ($dependsname, $dependsentry);
                my $obj = $cb->module_tree($dep); 

                unless( $obj ) {
                    cp_error( loc( "Couldn't find module object for prerequisite ".
                                "'%1' -- skipping", $dep ) );
                    # XXX we remove it otherwise it'll cause trouble again
                    delete($deps->{$dep});
                    $self->status->prereqs($deps);
                    next;
                };

                ### if the prereq requires any specific version, we'll assume
                ### the one we can provide, otherwise, we'll set it to undef,
                ### marking 'any'
                push @depends, [$obj, ($depver ? $obj->version : undef) ];
                ## for the PKGBUILD we reproduce the requirement if any:
                if ( package_is_in_arch_repos($obj) ) {
                    $dependsname = $perl_abs{$obj->package_name}{arch_package} ;
                }
                else {
                    $dependsname = &archpkgname($obj->package_name) ;
                };
                $dependsentry = $dependsname . $depverf;
                push @archdepends, $dependsentry; 
            }
        }
    };
    my %seendep = ();
    foreach (@archdepends) {
        $seendep{$_}++;
    };
    @archdepends = keys %seendep;
    
    push @archdepends, "perl" unless (@archdepends);
	  
	my $dependsline = join("\' \'", @archdepends);
    
    my @makedepends;
    my @archmakedepends;
    {   #my $bdeps = $dist->status->make_prereqs;
        #for my $bdep ( sort keys %$bdeps ) {}
        #	unless ( exists $makeprereqs{perl} ) {
        #	push @archmakedepends, 'perl' 
        #};
        
        for my $bdep ( sort keys %makeprereqs ) {
            my $obj = $cb->module_tree($bdep);

            unless( $obj ) {
                cp_error( loc( "Couldn't find module object for build prerequisite ".
                            "'%1' -- skipping", $bdep ) );
                next;
            }

            ### no point in listing prereqs that are IN the perl core
            ### themselves
            next if $obj->package_is_perl_core;

            ### We skip processing 'Test::*' modules if $skiptest is true
            next if ($skiptest and $bdep =~ m/^Test::/) ;

            ### if the prereq requires any specific version, we'll assume
            ### the one we can provide, otherwise, we'll set it to undef,
            ### marking 'any'
            my $bdepver = $makeprereqs{$bdep} ;
            push @makedepends, [$obj, ($bdepver ? $obj->version : undef) ];
            ## for the PKGBUILD we reproduce the requirement if any:
            my $bdepverf = $bdepver ? ">=$bdepver" : "";
			my $bdependsname;
			if ( package_is_in_arch_repos($obj) ) {
				$bdependsname = $perl_abs{$obj->package_name}{arch_package} 
			}
			else {
				$bdependsname = &archpkgname($obj->package_name) 
			};
            my $mdependsentry = $bdependsname . $bdepverf;

            push @archmakedepends, $mdependsentry; 
        }
    };
    
    if ($installtype =~ /build/i and $] < 5.01)  {
        push @archmakedepends, "perl-module-build"
    };
    my %seenmdep = ();
    foreach (@archmakedepends) {
        $seenmdep{$_}++;
    };
    @archmakedepends = keys %seenmdep;

    my $makedependsline = join('\' \'', @archmakedepends);
#   my $perlmmdefault = $ENV{'PERL_MM_USE_DEFAULT'} ;

	print PB <<"HERE"
# This PKGBUILD was generated by cpan4pacman via CPANPLUS::Dist::Pacman
# Contributor: $maintainer

pkgname=$archpkgname
_cpanname=$pkgname
$pkgverline
pkgrel=$pkgrel
pkgdesc="$pkgdesc"
arch=($pkgarch)
url="$pkgurl"
license=('GPL' 'PerlArtistic')
depends=('$dependsline')
HERE
    ;
    unless ($makedependsline eq '') {
        print PB <<"HERE"
makedepends=('$makedependsline')
HERE
    }
    ; 
    print PB <<"HERE"
options=('!emptydirs')
source=($pkgsource) 
md5sums=('$pkgmd5')

build() {
  cd \$srcdir/\${_cpanname}-\${$pkgverstring}
  sed -i '/^auto_install/d' Makefile.PL
HERE
    ;  
if ($] < 5.01) {
    if ($installtype =~ /build/i) {
        print PB <<"HERE"
  eval `perl -V:archname`
  perl Build.PL destdir=\$pkgdir \\
       --install_path lib=/usr/lib/perl5/site_perl/current \\
       --install_path arch=/usr/lib/perl5/site_perl/current/\${archname}
  perl Build
  perl Build install
HERE
    }
    else {
	    print PB <<"HERE"
  eval `perl -V:archname`
  PERL_MM_USE_DEFAULT=1 perl Makefile.PL \\
      INSTALLARCHLIB=/usr/lib/perl5/current/\${archname} \\
      INSTALLSITELIB=/usr/lib/perl5/site_perl/current \\
      INSTALLSITEARCH=/usr/lib/perl5/site_perl/current/\${archname}
  make || return 1
  make DESTDIR=\$pkgdir install || return 1
HERE
        }
}
else {
    ## THIS IS FOR NEW PERL POLICY starting with version 5.10.0
    if ($installtype =~ /build/i) {
        print PB <<"HERE"
  perl Build.PL installdirs=vendor destdir=\$pkgdir || return 1    
  perl Build || return 1
  perl Build install || return 1
HERE
    }
    else {
	    print PB <<"HERE"
  PERL_MM_USE_DEFAULT=1 perl Makefile.PL INSTALLDIRS=vendor || return 1 
  make || return 1
  make install DESTDIR=\$pkgdir || return 1
HERE
        }
} ;
unless ($skiptest) {
    print PB <<"HERE"
  make test || return 1
HERE
};
print PB <<"HERE"
  find \$pkgdir -name '.packlist' -delete
  find \$pkgdir -name '*.pod' -delete
}
HERE
;
close PB;

    ### this is required! set the status accessor that tells us where you
    ### created the packagedir
    $dist->status->distdir( $makepkgdir ); 

    return $dist->status->prepared( 'TRUE' ? 1 : 0 );
}


### actually create the dist target required.
sub create {
    ### just in case you already did a create call for this module object
    ### just via a different dist object
    my $dist = shift;
    my $self = $dist->parent;
    $dist    = $self->status->dist   if      $self->status->dist;
    $self->status->dist( $dist )     unless  $self->status->dist;

    my $cb   = $self->parent;
    my $conf = $cb->configure_object;
    my $force = $conf->get_conf('force');
	my $clean = $ARCHPKGCONF{'clean'};

	print "Now calling makepkg ...\n";
    ### other code here ###
    # XXX SIMPLY CALL makepkg on the PKGBUILD !
	# go to builddir and
	my $makepkgdir = $dist->status->distdir;
	chdir($makepkgdir);

	my $makepkgflags = $ARCHPKGCONF{'makepkgflags'};
	$makepkgflags .= " --clean" if $clean;
	$makepkgflags .= " --force" if $force;
	# useless or dangerous OPTIONS TO STRIP OUT of $makepkgflags : 
	# -i --install , -g --genmd5, -h --help, -p <buildscript> , -o --nobuild
	$makepkgflags =~ s/\b(-[A-Za-z]*i|--install)//g;
	$makepkgflags =~ s/\b(-[A-Za-z]*h|--help)//g;
	$makepkgflags =~ s/\b(-[A-Za-z]*g|--genmd5)//g;
	$makepkgflags =~ s/\b-p\s*\S+\b//g;
	$makepkgflags =~ s/\b(-[A-Za-z]*o|--nobuild)//g;
	my $mpcmd = "/usr/bin/makepkg $makepkgflags";
	my @makepkgbuf = run( command => $mpcmd, verbose => 0 ); 	
	
    ### this is required! set the status accessor that tells us where you
    ### created the package
    my $makepkg_dest = `source /etc/makepkg.conf && echo \$PKGDEST`;
#    my $makepkg_dest;
    #   my $makepkg_conf = open (MAKEPKG_CONF, '</etc/makepkg.conf') or cp_msg("Cannot open /etc/makepkg.conf");
#    while (<MAKEPKG_CONF>) {
#        next unless /^export PKGDEST/;
#        my @tmp = split /=/;
#        $makepkg_dest = pop @tmp;
#    };
#	close MAKEPKG_CONF;
    
    $dist->status->dist( $makepkg_dest );    
    return $dist->status->created( $makepkgbuf[0] ? 1 : 0 );
}

### takes care of the actual installation of the created dist
### you will need to require that 'create' has been run before
sub install {
    ### just in case you already did a create call for this module object
    ### just via a different dist object
    my $dist = shift;
    my $self = $dist->parent;
    $dist    = $self->status->dist   if      $self->status->dist;
    $self->status->dist( $dist )     unless  $self->status->dist;

#    my $cb   = $self->parent;
#    my $conf = $cb->configure_object;
#    my %hash = @_;

	print "Please first carefully check the package you have just built\nbefore installing it with pacman!\n";

    return $dist->status->installed( 0 );
}

1;

# Local variables:
# c-indentation-style: bsd
# c-basic-offset: 4
# indent-tabs-mode: nil
# End:
# vim: expandtab shiftwidth=4:
